<?php

namespace Bixo\ImportExport\Http\Controllers;

use Exception;
use Bixo\ImportExport\Actions\ImportExportWorkflow;
use Bixo\ImportExport\Http\Requests\ImportExportWorkflowRequest;
use Bixo\ImportExport\Http\Resources\ImportExportResource;
use Bixo\ImportExport\Models\ImportExport;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class ImportExportWorkflowController extends BaseController
{
    /**
     * Action controller function for import_export.
     *
     * @param Model $import_export
     * @param action next action for the import_export.
     *
     * @return Response
     */
    public function __invoke(ImportExportWorkflowRequest $request, ImportExport $import_export, $trasition)
    {
        try {
            $request = $request->all();
            $import_export = ImportExportWorkflow::run($trasition, $import_export, $request);
            $data = new ImportExportResource($import_export);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('import_export::import_export.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('import_export/import_export/' . $import_export->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('import_export/import_export/' . $import_export->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the import_export and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $import_export
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $import_export, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $import_export = ImportExport::findBySignedId($import_export);
        $transitions = $this->transitions($import_export, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('import_export::import_export.name') . ' ' . trans('Approvals'))
            ->view('import_export::public.import_export.workflow')
            ->layout('mobile')
            ->data(compact('import_export', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the import_export.
     *
     * @param PublicRequest $request
     * @param string $import_export
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $import_export, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $import_export = ImportExport::findBySignedId($import_export);
        $user = user();
        $import_export = ImportExportWorkflow::run($transition, $import_export, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($import_export, $user)
    {
        $transitions = [];
        foreach ($import_export->workflow()->transitions($import_export) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.import_export.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'import_export' => $import_export->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $import_export->workflow()->form($value);
            $array['label'] = trans('import_export::import_export.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
